% This fucntion produce the estimates of a VAR system with different lags
% and different trend specification
% trend(i) = 1 : with time trend, 
% trend(i) = 0 : no time trend
% Y{i,1} is data organized in ROW vectors;
% trend and L should be column vectors.
% y{i,1} = c1 + A1x1 + A2x2 + e1t
% L(i) is lags in Y{i,1};
% Following Hamilton(1994) p310-312, we can accomplish by running
% regression equation by equation
% Then rearrange to the VAR system as VAR(L), L = max(L{i})
% Yt = BXt + et, X = [const yt-1 yt-2 ... yt-L];
% S = et*et'/T;

% Note: The size of YY, L, trend much match, and length(YY)-L must match as
% well
function [B S et] = MLEVAR(YY,L,trend)
K   = size(YY,1); % number of blocks in the data
nn  = zeros(K,1); % number of variables in each block
y   = {};
X0  = [];
for k = 1:K
    nn(k,1) = size(YY{k,1},1);
    y{k,1} = YY{k,1}(:,L(k)+1:end);    
    tempx  = [];
    for i = 1:L(k)
        tempx = [tempx; YY{k,1}(:,L(k)-(i-1):end-i)];
    end;
    X0 = [X0;tempx];
end
T   = length(y{1,1});
ttrend  = 1:T;
n   = sum(nn);

et = [];
for k = 1:K
    clear x;
    if trend(k) == 0
        x   = [ones(T,1)'; X0];
    else
        x   = [ones(T,1)'; ttrend; X0];
    end;
    b_hat0{k,1} = y{k,1}*x'*inv(x*x'); % n(k) row vectors
    et = [et; y{k,1}-b_hat0{k,1}*x];     
end;

S   = et*et'/T; % variance of error

% Rearrange the coefficients
B0 = []; % coefficients on constant and trend if any, each row for each equation
B1 = [];
cn = [0; cumsum(nn.*L)];
LL = max(L);
B = {};
for k = 1:K
    if trend(k) == 0 
        B0 = [B0; b_hat0{k,1}(:,1) zeros(nn(k),1)];
        B1 = [B1; b_hat0{k,1}(:,2:end)];
    else
        B0 = [B0; b_hat0{k,1}(:,1:2)];
        B1 = [B1; b_hat0{k,1}(:,3:end)];
    end;
end;
for k = 1:K
    if L(k)< LL
        B{k,1} = [B1(:,1+cn(k):cn(k+1)) zeros(n,nn(k)*(LL-L(k)))];
    else
        B{k,1} = [B1(:,1+cn(k):cn(k+1))];
    end;
end;


b_hat = [];
for i = 1:LL
    for k = 1:K
        b_hat = [b_hat B{k,1}(:,(i-1)*nn(k)+1:i*nn(k))];   
    end;
end;

B   = [B0 b_hat];
