% This fucntion produce the estimates of a VAR system with block exogeneity (Granger Causality Restriction)
% allow for difference in lags and with and without trend in different
% blocks
% trend(i) = 1 : with time trend, 
% trend(i) = 0 : no time trend
% trend and L should be column vectors.
% YY is array of array
% YY{1,1} is the first block of blocks with different lags and trends
% YY{2,1} is the second block of blocks with different lags and trends
% YY{i,1}{j,1} are data organized in ROW vectors;
% y1 = c1 + A1x1 + A2x2 + e1t
% y2 = c2 + B1x2 + B2x2 + e2t
% Restriction is that y2 does not Granger Cause y1, A2 = 0;
% L(i,1) is lags in YY{i,1} and L(i,2) is lags in YY{i,2};
% Following Hamilton(1994) p310-312, we need to run the following two
% regression
% y1 = c1 + A1x1 +  e1t
% y2 = d + D0*y1 + D1*x1 + D2*x2 + v2t
% then convert to c, B1 B2 as following Hamilton(1994) p310-312
% The result is rearrange to the VAR system as VAR(L)
% Yt = BXt + et, X = [const yt-1 yt-2 ... yt-L];
% S = et*et'/T;

% Note: The size of YY, L, trend much match, and length(YY)-L must match as
% well
function [B S et] = MLEVARGC(YY,L,trend)
for i = 1:2
    K(i,1)  = size(YY{i,1},1); % number of blocks in the data
    nn{i,1} = zeros(K(i,1),1); % number of variables in each block
    X0{i,1} = [];
    for k = 1:K(i,1)
        nn{i,1}(k,1) = size(YY{i,1}{k,1},1);
        y{i,k} = YY{i,1}{k,1}(:,L{i,1}(k)+1:end);    
        tempx  = [];
        for j = 1:L{i,1}(k)
            tempx = [tempx; YY{i,1}{k,1}(:,L{i,1}(k)-(j-1):end-j)];
        end;
        X0{i,1} = [X0{i,1};tempx];
    end;    
    cn{i,1} = [0; cumsum(nn{i,1}.*L{i,1})]; % cum lags*#variables    
end;
T   = length(y{1,1}); % this number should be same for all blocks.

ttrend  = 1:T;

% Block one regress on its own lags
n1      = sum(nn{1,1}); % total number of variables in the first block
et{1,1} = [];
y1      = [];
for k = 1:K(1,1)
    clear x;
    y1  = [y1; y{1,k}]; % all ys in block one to be used in the second block regression
    if trend{1,1}(k) == 0
        x   = [ones(T,1)'; X0{1,1}];
    else
        x   = [ones(T,1)'; ttrend; X0{1,1}];
    end;
    b_hat0{1,k} = y{1,k}*x'*inv(x*x'); % n(k) row vectors
    et{1,1} = [et{1,1}; y{1,k}-b_hat0{1,k}*x];     
end;
S0{1,1}   = et{1,1}*et{1,1}'/T; % variance of error

% Block Two regress on its own lags, and lags of the variables in the first
% block and the contemporous variables in the first block.
n2      = sum(nn{2,1}); % total number of variables in the first block
et{2,1} = [];
for k = 1:K(2,1)
    clear x;
    if trend{2,1}(k) == 0
        x   = [ones(T,1)'; y1; X0{1,1}; X0{2,1}];
    else
        x   = [ones(T,1)'; ttrend; y1; X0{1,1}; X0{2,1}];
    end;
    b_hat0{2,k} = y{2,k}*x'*inv(x*x'); % n(k) row vectors
    et{2,1} = [et{2,1}; y{2,k}-b_hat0{2,k}*x];     
end;
S0{2,1}   = et{2,1}*et{2,1}'/T; % variance of error

% Calculate the coefficient we need 
c1 = [];
A1 = [];
for k = 1:K(1,1)
    if trend{1,1}(k)==0
        c1 = [c1; b_hat0{1,k}(:,1) zeros(nn{1,1}(k),1)];
    else
        c1 = [c1; b_hat0{1,k}(:,1:2)];
    end;
    A1 = [A1; b_hat0{1,k}(:,2+trend{1,1}(k):end)];
end;
d = [];
D0 = [];
D1 = [];
D2 = [];
for k = 1:K(2,1)
    if trend{2,1}(k)==0
        d = [d; b_hat0{2,k}(:,1) zeros(nn{2,1}(k),1)];
    else
        d = [d; b_hat0{2,k}(:,1:2)];
    end;    
    ST   = 1+trend{1,1}(k);    
    D0   = [D0; b_hat0{2,k}(:,1+ST:ST+n1)];
    D1   = [D1; b_hat0{2,k}(:,1+ST+n1:ST+n1+(nn{1,1}'*L{1,1}))];
    D2   = [D2; b_hat0{2,k}(:,1+ST+n1+(nn{1,1}'*L{1,1}):end)];
end;

H   = S0{2,1};
S11 = S0{1,1};
S21 = D0*S11;
S22 = H + S21*inv(S11)*S21';
S   = [S11 S21'; S21 S22];

c2  = d + S21*inv(S11)*c1;
B1  = D1 + S21*inv(S11)*A1;
B2  = D2;

% Rearrange the coefficients
B0 = [c1;c2]; % coefficients on constant and trend if any, each row for each equation

LL = max(max(L{1,1}),max(L{2,1}));
for k = 1:K(1,1) % coefficient on variables in block one
    if L{1,1}(k)< LL
        B{k,1} = [[A1(:,1+cn{1,1}(k):cn{1,1}(k+1));B1(:,1+cn{1,1}(k):cn{1,1}(k+1))] zeros(n1+n2,nn{1,1}(k)*(LL-L{1,1}(k)))];
    else
        B{k,1} = [A1(:,1+cn{1,1}(k):cn{1,1}(k+1));B1(:,1+cn{1,1}(k):cn{1,1}(k+1))];
    end;
end;

for k = 1:K(2,1) % coefficient on variables in block two
    if L{2,1}(k)< LL
        B{k+K(1,1),1} = [zeros(n1,nn{2,1}(k)*LL);[B2(:,1+cn{2,1}(k):cn{2,1}(k+1)) zeros(n2,nn{2,1}(k)*(LL-L{2,1}(k)))]];
    else
        B{k+K(1,1),1} = [zeros(n1,nn{2,1}(k)*LL); B2(:,1+cn{2,1}(k):cn{2,1}(k+1))];
    end;
end;

nnn = [nn{1,1};nn{2,1}];
b_hat = [];
for i = 1:LL
    for k = 1:sum(K)
        b_hat = [b_hat B{k,1}(:,(i-1)*nnn(k)+1:i*nnn(k))];   
    end;
end;

B   = [B0 b_hat];
