% This function gives the solution of value func (v1)and derivative of valuefunc (dv1) wrt rho at rho=1
% and linear dynamics of consumption as
% x(t) = G*x(t-1)+H*w(t), wt follow std norm dist.
% c(t+1)-c(t) = mc + Uc*x(t)+lambda0*w(t+1)
% lambdab is the discounted response of consumption to shocks
% lambdab = lambda0+beta*Uc*inv(I-G)*H
% result: 
% v(t) = v1(t) + (rho-1)*Dv1(t)=-1/2x(t)'*Ydrho*x(t)+Uvrho*x(t)+mvrho
% v1(t)= v1(t) = mv+Uv*x(t)
% Dv1(t) = -1/2x(t)'*Yd*x(t)-Udv*x(t)-mdv,  
% where Yd satisfies: beta*G'Yd*G - Yd + (1-beta)/beta*Uv*Uv'=0 (Lyapunov Equation) 

function [Yd Udv mdv Uv mv] = VALUEFUNC1(G, H, mc, Uc, lambda0, beta, gamma, rho)
tempG  = inv(eye(size(G))-beta*G);
lambdab = lambda0 + beta*Uc*tempG*H;
% value function when rho =1, v1 = mc+Uv*x_t
Uv     = beta*Uc*tempG;
mv     = beta/(1-beta)*(mc + (1-gamma)*(lambdab*lambdab')/2);
% second solve dv1 = -(x_t'*Yd*x_t/2+Udv*x_t+mdv)
A      = beta^0.5*G';
Q      = (1-beta)/beta*Uv'*Uv;
Yd     = dlyap(A,Q); 
Udv    = ((1-beta)/beta*mv*Uv + beta*(1-gamma)*lambdab*H'*Yd*G)*tempG;
mdv    = mv^2/(2*beta)+beta/(2*(1-beta))*(trace(H'*Yd*H)+2*(1-gamma)*Udv*H*lambdab'+(1-gamma)^2*lambdab*H'*Yd*H*lambdab');