%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright Hansen, Heaton, Li (2007)                                   
% Last revised May 2008
% This program estimates dividends dynamics in alternative specification
% and produces Figure 8
% Model: in first difference
% Mode2: level with no time trend
% Mode3: level with time trend
% Note: In all of the these systems we restrict dividends do not Granger
% cause consumption and earning, and consumption and earning are
% cointegrated.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all;
close all;
load longrun.mat;
N = size(d,2); % number of portfolios
logc = log(c);
loge = log(e);
logd = log(d);
gc = diff(logc);
ge = diff(loge);
gd = diff(logd);
logdc = logd -logc*ones(1,N);
logec = loge -logc;
L = 5;
n = 3;
T = length(logec)-L;

%% Step 1: VAR in in [gc e-c]
n1 = 2;
y{1,1} = gc';
y{2,1} = logec';
[B S11 et] = MLEVAR(y, [L-1;L], [0;0]);
% orthogonize the shocks as permanent shock and temporary shock
A0     = B(:,3:end);
A1    = DISCSUM(A0,1,L);
temp  = inv(eye(2)-A1);
tempV = (chol(temp*S11*temp'))';
V    = (eye(2)-A1)*tempV; % Blanchard-Quach Decomposition
et2   = inv(V)*et;

% rearrange the coefficient in the level specification
B1 = zeros(size(B));
B1(1,1:4) = [B(1,1:2) 1+A0(1,1)-A0(1,2) A0(1,2)];
B1(2,1:4) = [B(2,1:2) A0(2,1)-A0(2,2) A0(2,2)];
for i = 2:L
    B1(:,2+(i-1)*2+1:2+i*2) = [-A0(:,(i-2)*2+1)+A0(:,(i-1)*2+1)-A0(:,i*2) A0(:,i*2)];
end;
B1(2,:) = sum(B1);
V1 = [V(1,:); V(1,:)+V(2,:)];

G = [B1(:,3:end);[eye((L-1)*n1) zeros((L-1)*n1,n1)]];
H = [V1; zeros((L-1)*n1, n1)];
uc = [1 0 zeros(1,(L-1)*n1)];  

TT = 80;
for t = 1:TT
    IR(t,:) = uc*G^(t-1)*H;
end;

%% Step 2: append the dividend dynamics to the above system in consumption and earnings
% Model 1: dividend in first difference
ud = [0 0 1 zeros(1,(L-1)*n)]; 
clear y;
y1{1,1}     = gc';
y1{2,1}     = logec';
y{1,1}      = y1;
LL{1,1}     = [L-1;L]; 
LL{2,1}     = L-1;
trend{1,1}  = [0;0];
trend{2,1}  = 0;
T = length(logc)-L; % sample size
n = 3; % number of variables in each VAR
for i = 1:N
    y{2,1}  = {gd(:,i)'};
    [Bd Sd etd] = MLEVARGC(y, LL, trend);
    mx  = [Bd(:,1);zeros((L-1)*n, 1)]; % constant
    Ad  = [Bd(:,3:end)]; % coefficient on laggs
    A1d = DISCSUM(Ad,1,L); % discounted sum of matrix
    temp   = inv(eye(n)-A1d);
    tempV  = (chol(temp*Sd*temp'))';
    Vd     = (eye(n)-A1d)*tempV;
    G2{1,i} = [Ad;[eye((L-1)*n) zeros((L-1)*n,n)]];
    H2{1,i} = [Vd; zeros((L-1)*n, n)];
    for t = 1:TT;
        tempIRd(t,:) = ud*G2{1,i}^(t-1)*H2{1,i};
    end;
    IRd{1,i} = cumsum(tempIRd);
end;


% Model 2: level specification without trend 
TT = 80;
clear y;
for i = 1:N
    y = logd(L+1:end,i);
    x1 = [logc loge];
    x2 = logd(:,i);
    X = [];
    X = [X ones(T,1) logc(L+1:end,1) loge(L+1:end,1)];
    for k = 1:L
        X = [X x1(L-k+1:end-k,:)];        
    end;
    for k = 1:L
        X = [X x2(L-k+1:end-k,:)];        
    end;    
    b_hat = (inv(X'*X)*X'*y)';
    e_hat = y-X*b_hat'; % e_hat is orthogonal to the shocks in first two equations
    s22   = e_hat'*e_hat/T;
    d0    = b_hat(1,1); % constant
    D0    = b_hat(1,2:1+n1); % coefficient on contemparous c and e.
    D1    = b_hat(1,2+n1:3+n1*L); % cofficient on lagged c and e
    D2    = b_hat(1,2+n1*(L+1):end); % coefficient on lagged d
    S21   = D0*S11;
    B2    = zeros(1,2+n*L);
    B2(1,1)    = d0 + D0*B1(:,1);
    B2(1,3:12) = D1 + D0*B1(:,3:end);
    B2(1,13:end) = D2; 
    tempB = [B1 zeros(n1,L); B2];
    A  = zeros(n,2+n*L);
    A(:,1:2) = tempB(:,1:2);
    for k = 1:L
        A(:,2+(k-1)*n+1:2+k*n) = [tempB(:,2+(k-1)*n1+1:2+k*n1) tempB(:,2+L*n1+k)];
    end;
    G2{2,i}  = [A(:,3:end);[eye((L-1)*n) zeros((L-1)*n,n)]];
    V2 = [V zeros(2,1);D0*V sqrt(s22)]; 
    H2{2,i} = [V2; zeros((L-1)*n, n)];
    for t = 1:TT;
        IRd{2,i}(t,:) = ud*G2{2,i}^(t-1)*H2{2,i};
    end;
end;

% Model 3: level specification with trend
ud = [0 0 1 zeros(1,(L-1)*n)];  
TT = 80;
for i = 1:N
    y = logd(L+1:end,i);
    x1 = [logc loge];
    x2 = logd(:,i);
    X = [];
    trend = (1:T)';
    X = [X ones(T,1) trend logc(L+1:end,1) loge(L+1:end,1)];
    for k = 1:L
        X = [X x1(L-k+1:end-k,:)];        
    end;
    for k = 1:L
        X = [X x2(L-k+1:end-k,:)];        
    end;    
    b_hat = (inv(X'*X)*X'*y)';
    e_hat = y-X*b_hat'; % e_hat is orthogonal to the shocks in first two equations
    s22   = e_hat'*e_hat/T;
    d0    = b_hat(1,1); % constant
    D0    = b_hat(1,3:2+n1); % coefficient on contemparous c and e.
    D1    = b_hat(1,3+n1:2+n1*(L+1)); % cofficient on lagged c and e
    D2    = b_hat(1,3+n1*(L+1):end); % coefficient on lagged d
    S21   = D0*S11;
    B2    = zeros(1,2+n*L);
    B2(1,1)    = d0 + D0*B1(:,1);
    B2(1,3:12) = D1 + D0*B1(:,3:end);
    B2(1,13:end) = D2; 
    tempB = [B1 zeros(n1,L); B2];
    A  = zeros(n,2+n*L);
    A(:,1:2) = tempB(:,1:2);
    for k = 1:L
        A(:,2+(k-1)*n+1:2+k*n) = [tempB(:,2+(k-1)*n1+1:2+k*n1) tempB(:,2+L*n1+k)];
    end;
    G2{3,i}  = [A(:,3:end);[eye((L-1)*n) zeros((L-1)*n,n)]];
    V2 = [V zeros(2,1);D0*V sqrt(s22)]; 
    H2{3,i} = [V2; zeros((L-1)*n, n)];
    for t = 1:TT;
        IRd{3,i}(t,:) = ud*G2{3,i}^(t-1)*H2{3,i};
    end;
end;

%figure(1)
%plot([IR(:,1) -IR(:,2)]);
%title('Consumption response to permanent and temporary shocks')
%ylim([-0.005, 0.015]);

figure
modelindex = ['First difference        '; 
              'Level without time trend';
              'Level with time trend   '];
for k = 1:3
    subplot(2,2,k)
    plot(1:TT,IRd{k,5}(:,1),'b-',1:TT,IRd{k,1}(:,1),'r-.')
    title(modelindex(k,:))
    ylim([-0.03,0.06])
    xlim([1,TT])
    xlabel('Horizon in Quarters')
end;
legend('Value Portfolio','Growth Portfolio')

print -depsc figure8;

