%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright Hansen, Heaton, Li (2007)                                   
% Last revised May 2008
% This program estimates the dynamics of system [c_t-c_(t-1) e_t-c_(t-1) d_t-d_(t-1)]
% with restriction that dt does not Granger Cause consumption and earning
% and calculation longrun price in general case of rho NOT equal to 1
% 1. plot long-run risk free rate (Figure 6)
% 2. plot expected return (Figure 2)
% 3. plot long-run risk premium of consumption flow (Figure 7)
% 4. Table 2
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;
close all;
load longrun.mat;
N = size(d,2); % number of portfolios
logc = log(c);
loge = log(e);
logd = log(d);
gc = diff(logc);
ge = diff(loge);
gd = diff(logd);
logdc = logd -logc*ones(1,N);
logec = loge -logc;
L = 5; % number of lags
n = 3;
T = length(logec)-L;
b = ((0.8:0.01:1).^0.25)'; % when k = 18 b = 0.97^0.25
%b = 0.97^0.25;
gamma = [5 20]';
rho = [2/3 1 3/2]';

% Step 1; Estimate model
B  = {}; % coefficient
S  = {}; % var-cov matrix of error
et = {}; % error

% VAR in with GC; y= [gc e-c gd] 
y1{1,1}     = gc';
y1{2,1}     = logec';
y{1,1}      = y1;
LL{1,1}     = [L-1;L]; 
LL{2,1}     = L-1;
trend{1,1}  = [0;0];
trend{2,1}  = 0;

T = length(logc)-L; % sample size
n = 3; % number of variables in each VAR
logL = zeros(N,1);

for i = 1:N
    y{2,1}  = {gd(:,i)'};
    [B{i} S{i} et{i}] = MLEVARGC(y, LL, trend);
    logL(i,1) = (-T/2)*(n*log(2*pi)+n+log(det(S{i})));
end;

% Step 2; rearrange the systems to VAR(1)
% factorize the variance-covariance matrix to define different shocks
% V1: cholesky decompostion of variance-covariance matrix
% V2: BQ decomposition, lambda1 (limiting response) is lower triagular, i.e. permanent and
% temporary shocks
uc  = [1 0 0 zeros(1,(L-1)*n)];  
ue  = [0 1 0 zeros(1,(L-1)*n)];
ud  = [0 0 1 zeros(1,(L-1)*n)];
for i = 1:N
    mx{i}  = [B{i}(:,1);zeros((L-1)*n, 1)]; % constant
    A{i}   = [B{i}(:,3:end)]; % coefficient on laggs
    A1{i}  = DISCSUM(A{i},1,L); % discounted sum of matrix
    V{i,1} = chol(S{i})'; % cholasky decomposition of variance-cov
    temp   = inv(eye(n)-A1{i});
    tempV  = (chol(temp*S{i}*temp'))';
    V{i,2} = (eye(n)-A1{i})*tempV;
    G{i}   = [A{i};[eye((L-1)*n) zeros((L-1)*n,n)]];
    H{i,1} = [V{i,1}; zeros((L-1)*n, n)];
    H{i,2} = [V{i,2}; zeros((L-1)*n, n)];   
end

% Calculate Uc, mc and lambda0 due the Granger Causality restriction, 
% mc, Uc, Ue are same for every portfolio
mc = uc*inv(eye(size(G{1}))-G{1})*mx{1};
Uc = uc*G{1};
Ue = ue*(G{1}-eye(size(G{1})))+uc*G{1};
for i = 1:N
    Ud{i} = ud*G{i};
    md{i} = ud*inv(eye(size(G{i}))-G{i})*mx{i};
    for k = 1:2
        iota0{i,k}=ud*H{i,k};
        iotab{i,k} = []; % discounted response of dividends
        for jb = 1:length(b)
            tempG = inv(eye(size(G{i}))-b(jb)*G{i});
            iotab{i,k} = [iotab{i,k}; ud*tempG*H{i,k}];            
        end;        
    end;
end;

% calculate lambda0 and lambda(beta) same for every portfolio
% risk free rate and price of consumption when rho = 1
for k = 1:2 % different shock identification
    lambda0{k} = uc*H{1,k};
    lambdab{k} = []; % discounted response of consumption
    for jb = 1:length(b)
        tempG = inv(eye(size(G{1}))-b(jb)*G{1});
        lambdab{k} = [lambdab{k}; uc*tempG*H{1,k}];        
    end;
    normb{k} = sqrt(sum(lambdab{k}.^2,2)); % norm of lambdab
    lambda1{k} = lambdab{k}(end,:);
    rpc{k} = lambda1{k}*lambda1{k}'+(gamma-1)*(lambda1{k}*lambdab{k}'); % risk premium of consumption
    for j = 1:2
        pricec{k,j} = lambda1{k}(1,j)+(gamma-1)*(lambdab{k}(:,j)'); % risk price vector
    end;
end;


% Calculate long-run risk free rate, expected excess return 
lrgr = zeros(N,1); 
dnu1 = {};
for i = 1:N
        % long-run growth rate = md(i)+iota1*iota1'/2;
        lrgr(i,1) = md{i}+iotab{i,1}(end,:)*iotab{i,1}(end,:)'/2;
    for k = 1:2
        dnu1{i,k} = zeros(length(b),length(gamma));
        for jg = 1:length(gamma)    
            for jb = 1:length(b)-1
                % value function
                [Yd Udv mdv Uv mv] = VALUEFUNC1(G{i}, H{i,k}, mc, Uc, lambda0{k}, b(jb), gamma(jg));
                % SDF1
                [ms Us kesi0] = SDF1(G{i}, H{i,k}, mc, Uc, lambda0{k}, b(jb), gamma(jg));
                % DSDF1
                [Theta0 Theta1 theta0 theta1 theta2] = DSDF1(G{i}, H{i,k}, lambdab{k}(jb,:), mv, Uv, Udv, Yd, b(jb), gamma(jg));
                % derivative of long-run return when rho = 1
                tempG = inv(eye(size(G{i}))-G{i});
                wbar = -Us*tempG; % eigenfunction when rho =1 
                mw   = (iotab{i,k}(end,:)+kesi0-wbar*H{i,k})';
                mx   = tempG*H{i,k}*mw;
                dnu1{i,k}(jb,jg) = -(0.5*mw'*Theta0*mw+0.5*trace(Theta0)+mw'*Theta1*mx+theta0+theta1*mx+theta2*mw);
                for jr = 1:length(rho)
                    [Ydrho Uvrho mvrho] = VALUEFUNC(Yd, Udv, mdv, Uv, mv,rho(jr));
                    [Theta0rho Theta1rho msrho Usrho kesi0rho] = SDF(Theta0, Theta1, theta0, theta1, theta2, ms, Us, kesi0, rho(jr));
                    % longrun return
                    [W w nu] = PRICEOPEBAR(G{i},H{i,k},Theta0rho,Theta1rho,msrho,Usrho,kesi0rho,md{i},iotab{i,k}(end,:));
                    % long-run risk free rate
                    [temprf pistar PIstar] = RETOPE(G{i},H{i,k},W,w,Theta0rho,Theta1rho,msrho,Usrho,kesi0rho);
                    lrret{i,k,jg}(jb,jr) = nu+lrgr(i,1); % long-run return
                    lrrf{i,k,jg}(jb,jr) =  temprf;       % long-run risk free rate  
                    lrrp{i,k,jg}(jb,jr) = nu+lrgr(i,1)-temprf;  % long-run risk premium (excess return)                 
                end;
            end;
        end;
    end;
end;

% Return decomposition by horizons fix beta = 0.97^.025, gamma = 5 or 20,
% rho = 2/3 1 or 3/2
jb = 18; %beta = 0.97^0.25;
k  = 1; % use cholesky decomposition for shocks 
TT = 100;
for jg = 1:length(gamma)
    for jr = 1:length(rho)        
        retj1{jg,jr} = zeros(N,TT); % Expected return by cash flow horizon 
        retj2{jg,jr} = zeros(N,TT); % Expected holding period return by horizon
        rfj{jg,jr} = zeros(1,TT); % risk free by horizon
        exretj{jg,jr} = zeros(N,TT); % expected excess return by horizon
        for i = 1:N
            [Yd Udv mdv Uv mv] = VALUEFUNC1(G{i}, H{i,k}, mc, Uc, lambda0{k}, b(jb), gamma(jg));
            [ms Us kesi0] = SDF1(G{i}, H{i,k}, mc, Uc, lambda0{k}, b(jb), gamma(jg));
            [Theta0 Theta1 theta0 theta1 theta2] = DSDF1(G{i}, H{i,k}, lambdab{k}(jb,:), mv, Uv, Udv, Yd, b(jb), gamma(jg));
            [Theta0rho Theta1rho msrho Usrho kesi0rho] = SDF(Theta0, Theta1, theta0, theta1, theta2, ms, Us, kesi0, rho(jr));
            % calculate risk free rate by horizon setting pi ( i.e. iota1) =0
            w0 = zeros(1,n*L);
            W0 = zeros(n*L);
            k0 = 0;
            wg0 = w0;
            Wg0 = W0;
            kg0 = k0;           
            if i ==1
                for t = 1:TT
                [W1 w1 k1] = PRICEOPE(W0,w0,k0,G{i},H{i,k},Theta0rho,Theta1rho,msrho,Usrho,kesi0rho,md{i},zeros(1,n));
                [Wg1 wg1 kg1] = PRICEOPE(Wg0,wg0,kg0,G{i},H{i,k},zeros(n),zeros(n,n*L),0,zeros(1,n*L),zeros(1,n),md{i},zeros(1,n));
                rfj{jg,jr}(i,t) = (kg1 - k1 )/t;                
                k0 = k1;
                w0 = w1;
                W0 = W1;
                Wg0 = Wg1;
                wg0 = wg1;
                kg0 = kg1;
                end;
            end;
            % calculate expected return and holding-period return by horizon
            w0 = Ud{i}*inv(eye(size(G{i}))-G{i});
            W0 = zeros(n*L);
            k0 = 0;
            wg0 = w0;
            Wg0 = W0;
            kg0 = k0;
            for t = 1:TT
                [W1 w1 k1] = PRICEOPE(W0,w0,k0,G{i},H{i,k},Theta0rho,Theta1rho,msrho,Usrho,kesi0rho,md{i},iotab{i,1}(end,:));
                [Wg1 wg1 kg1] = PRICEOPE(Wg0,wg0,kg0,G{i},H{i,k},zeros(n),zeros(n,n*L),0,zeros(1,n*L),zeros(1,n),md{i},iotab{i,1}(end,:));
                retj1{jg,jr}(i,t) = (kg1 - k1 )/t;
%               retj2{jg,jr}(i,t) = md{i} + k0 - k1 ;                
                retj2{jg,jr}(i,t) = kg1-kg0 - (k1 - k0) ;                
                exretj{jg,jr}(i,t) = retj1{jg,jr}(i,t)- rfj{jg,jr}(1,t);
                k0 = k1;
                w0 = w1;
                W0 = W1;
                Wg0 = Wg1;
                wg0 = wg1;
                kg0 = kg1;
            end;

        end;
    end;
end;


tt = 1:TT;
figure(1)
subplot(2,1,1)
plot(tt, retj1{2,2}(5,:)'*400,'-', tt, retj1{2,2}(6,:)'*400,'--', tt, retj1{2,2}(1,:)'*400,'-.');
title('Expected returns by cash flow horizon \gamma = 20 \rho = 1 \beta=0.97^0^.^2^5');
legend('Value Portfolio', 'Market Portfolio', 'Growth Portfolio',2)
ylim([4 12]);
xlim([0 100]);
subplot(2,1,2)
plot(tt, retj2{2,2}(5,:)'*400,'-', tt, retj2{2,2}(6,:)'*400,'--', tt, retj2{2,2}(1,:)'*400,'-.');
title('Holding period return \gamma = 20 \rho = 1 \beta=0.97^0^.^2^5');
ylim([4 12]);
xlim([0 100]);


figure(2)
subplot(2,1,1)
plot(tt,rfj{1,3}*400, '--',tt,rfj{1,2}*400,'-',tt,rfj{1,1}*400,'-.');
title('Logarithm of risk free returns \gamma = 5');
legend('\rho = 3/2', '\rho = 1', '\rho = 2/3',3);
xlabel('Horizon in Quarters');
ylim([0 12]);
xlim([0 100]);
subplot(2,1,2)
plot(tt,rfj{2,3}*400, '--',tt,rfj{2,2}*400,'-',tt,rfj{2,1}*400,'-.');
title('Logarithm of risk free returns \gamma = 20');
ylim([0 12]);
xlim([0 100]);
xlabel('Horizon in Quarters');

figure(3)
subplot(2,2,1)
plot(tt,exretj{1,1}(1,:)*400,'-.',tt,exretj{1,2}(1,:)*400,'-',tt,exretj{1,3}(1,:)*400,'--');
ylim([-1 4.5]);
title('portfolio 1 \gamma = 5');
legend('\rho = 2/3', '\rho = 1', '\rho = 3/2');
subplot(2,2,2)
plot(tt,exretj{1,1}(5,:)*400,'-.',tt,exretj{1,2}(5,:)*400,'-',tt,exretj{1,3}(5,:)*400,'--');
title('portfolio 5 \gamma = 5');
ylim([-1 4.5]);
subplot(2,2,3)
plot(tt,exretj{2,1}(1,:)*400,'-.',tt,exretj{2,2}(1,:)*400,'-',tt,exretj{2,3}(1,:)*400,'--');
ylim([-1 4.5]);
title('portfolio 1 \gamma = 20');
subplot(2,2,4)
plot(tt,exretj{2,1}(5,:)*400,'-.',tt,exretj{2,2}(5,:)*400,'-',tt,exretj{2,3}(5,:)*400,'--');
ylim([-1 4.5]);
title('portfolio 5 \gamma = 20');
print -depsc figure7;
% Produce Table 2 in HHL(2007)
% long-run rate of return (rh0=1), derivative (rho=1) and growth rate, 
% beta = 0.97^0.25
tempret1 = []; % gamma = 5
tempret2 = [];% gamma = 20
for i = 1:N
    tempret1 = [tempret1; lrret{i,1,1}(18,2) dnu1{i,1}(18,1)];
    tempret2 = [tempret2; lrret{i,1,2}(18,2) dnu1{i,1}(18,2)];    
end;
table2 = [[1:6 1:6]' [tempret1 lrgr(:,1);tempret2 lrgr(:,1)]*400 ]
longrunrf = [gamma(1) gamma(2);
    [lrrf{1,1,1}(18,:)' lrrf{1,1,2}(18,:)']*400]