%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright Hansen, Heaton, Li (2007)                                   
% Last revised May 2008
% This program simulates the posterior distribution for lambda(1) and lambda(0)
% in the system [c_t-c_(t-1) e_t-c_(t-1) d_t-d_(t-1)]
% restrict d_t not Granger Cause Ct and et
% VAR system is estimated by equation-by-equation OLS regression
% noninformative prior distribution for variance and OLS coefficient implies that
% posterior distribution for inverse of variance (percision) is chi-square 
% posterior distribution for OLS coefficients is normal
% result:
% figure 4: posterior histogram for the norm(gamma0) and figure(gamma1)
% Table 3: Posterior percentile of gamma1*iota1(pi)*400 except last row
% Note: norm of lambda(1) and lambda(0) are the same for different types of
% orthoganization of shocks. Here Sims orthoganization is used.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;
close all;
load longrun.mat;
logc = log(c);
loge = log(e);
logd = log(d);
%load cashflowffcusipcs.txt;
%logc = log(cashflowffcusipcs(:,5));
%loge = log(cashflowffcusipcs(:,6));
%logd = log(cashflowffcusipcs(:,[17:21 23]));
gc = diff(logc);
gd = diff(logd);
logec = loge -logc;
L = 5; % number of lags
T = length(logec)-L;
n = 2;
n2 = 1;

%Step 1: Estimate VAR with cointegration by OLS, y = [gc c-e gd]
y{1} = gc(L:end);
y{2} = logec(L+1:end);
X{1} = [];
for i = 1:L-1
    X{1} = [gc(i:end-(L-i),1) logec(i+1:end-(L-i),1) X{1}];
end;
X{1} = [X{1} logec(1:end-L,1)];
X{2} = [ones(T,1) X{1} y{1}];
X{1} = [ones(T,1) X{1}];
uc = [1 0 0 zeros(1,(L-1)*(n+1))];
ud = [0 0 1 zeros(1,(L-1)*(n+1))];

% estimate [gc c-e] system and stack it into VAR(L)
for k = 1:2
    XX{k}  = inv(X{k}'*X{k});
    b{k}   = inv(X{k}'*X{k})*X{k}'*y{k};
    err{k} = y{k} - X{k}*b{k};
    rss(k) = err{k}'*err{k};
    sig(k) = sqrt(rss(k)/T);    
end;

constant1 = [b{1}(1) b{2}(1)+b{1}(1)*b{2}(end)]';
tempB = [b{1}(2:end) b{2}(2:end-1)+b{1}(2:end)*b{2}(end)]';
B1 = [];
for i = 1:L-1
    B1 = [B1 tempB(:,(i-1)*n+1:i*n) zeros(n,1)];
end;
B1 = [B1 zeros(n,1) tempB(:,end) zeros(n,1)];
V1 = [sig(1) 0; sig(1)*b{2}(end) sig(2)];

prc = [];
for m =1:6 % portfolio index (1-5 and market)
    y{3} = gd(L:end,m);
    X{3} = [];
    for i = 1:L-1
        X{3} = [gc(i:end-(L-i),1) logec(i+1:end-(L-i),1) gd(i:end-(L-i),m) X{3}];
    end;
    X{3}   = [X{3} logec(1:end-L,1) y{1} y{2}];
    X{3}   = [ones(T,1) X{3}];
    XX{3}  = inv(X{3}'*X{3});
    b{3}   = inv(X{3}'*X{3})*X{3}'*y{3};
    err{3} = y{3} - X{3}*b{3};
    rss(3) = err{3}'*err{3};
    sig(3) = sqrt(rss(3)/T);    
    
    % Step 2: Stack the system into VAR(L)    
    constant2 = b{3}(1)+ b{3}(end-1:end)'*constant1;
    constant = [constant1;constant2];
    tempB2 = [b{3}(2:end-n-1)' 0 b{3}(end-n) 0];
    B2 = tempB2 + b{3}(end-1:end)'*B1;
    A = [B1;B2];    
    V2 = [b{3}(end-1:end)'*V1 sig(3)];
    V  = [V1 zeros(n,1); V2]; 

    % Step 3: rearrange the systems to VAR(1)
    G  = [A;[eye((L-1)*(n+1)) zeros((L-1)*(n+1),(n+1))]];
    H  = [V; zeros((L-1)*(n+1), (n+1))];

    % Step 4: Calculate gamma1 and iota1 (pi)
    tempG  = inv(eye(size(G))-G);
    gamma0 = uc*H;       
    gamma1 = uc*tempG*H; 
    iota1  = ud*tempG*H; 
    price(m)  = gamma1*iota1'*400; 
    
    
    % Monte Carlo Simulation
    knum=100000;

    simdata_price = zeros(knum,1); % norm of gamma1'*iota1
    
    ncheck = 1000;
    ntest = 0; 

    for k=1:knum
        for i = 1:3 %first and second equation
            scale = chi2rnd(T);
            newalpha = sqrt(scale/rss(i));
            mean = - newalpha*b{i};
            newbetasc = mvnrnd(mean, XX{i});
            newb{i} = - (newbetasc/newalpha)';
            newsig(i) = 1/newalpha;
        end
        tempB = [newb{1}(2:end) newb{2}(2:end-1)+newb{1}(2:end)*newb{2}(end)]';
        B1 = [];
        for i = 1:L-1
            B1 = [B1 tempB(:,(i-1)*n+1:i*n) zeros(n,1)];
        end;
        B1 = [B1 zeros(n,1) tempB(:,end) zeros(n,1)];
        tempB2 = [newb{3}(2:end-n-1)' 0 newb{3}(end-n) 0];
        B2 = tempB2 + newb{3}(end-1:end)'*B1;
        newA = [B1;B2];
        V1 = [newsig(1) 0; newsig(1)*newb{2}(end) newsig(2)];
        V2 = [newb{3}(end-1:end)'*V1 newsig(3)];
        newV  = [V1 zeros(n,1); V2]; 
        newG  = [newA;[eye((L-1)*(n+1)) zeros((L-1)*(n+1),(n+1))]];
        newH  = [newV; zeros((L-1)*(n+1), (n+1))];
    
%        check = max(abs(eig(newG)));
%        if check < .9999
         if max(abs(eig(newG))) < 1
            ntest = ntest +1;
            tempG = inv(eye(size(newG))-newG);            
            newgamma1 = uc*tempG*newH; 
            newiota1  = ud*tempG*newH;
            simdata_price(ntest,1) = newgamma1*newiota1'*400; 
            if m == 1 
                newgamma0 = uc*newH;
                simdata_lr(ntest,1) = norm(newgamma1);
                simdata_sr(ntest,1) = norm(newgamma0);
            end;
            if  ntest > ncheck
                ntest
                ncheck = ncheck + 1000;
            end
        end
    end
    simdata_price_c = simdata_price(1:ntest);
    prc = [prc; prctile(simdata_price_c,[5 25 50 75 95])];
end
simdata_lr_c    = simdata_lr(1:ntest);
simdata_sr_c    = simdata_sr(1:ntest);

% Plotting the norm of gamma0 and gamma1
close all
figure(1)
subplot(2,1,1)
[n,xout]=hist(simdata_sr_c,0.002:0.00001:0.008);
barrdiff = xout(2) - xout(1);
bar(xout,n/barrdiff/sum(n))
axis([0 0.01  0 2500])
hold 
medval = median(simdata_sr_c);
plot([medval,medval],[0, 10000])

subplot(2,1,2)
[n,xout]=hist(simdata_lr_c,0:0.001:1);
barrdiff = xout(2)-xout(1);
bar(xout,n/barrdiff/sum(n))
axis([0 0.1 0 140])
hold 
medval = median(simdata_lr_c);
plot([medval,medval],[0, 10000])

