%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright Hansen, Heaton, Li (2007)                                   
% Last revised May 2008
% This program estimate the dynamics of consumption and earning system
% and calculate longrun price in case of IES = 1
% produce figure 3 and figure 5   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all;
close all;
load longrun.mat;
logc = log(c(1:end));
loge = log(e(1:end));
logd = log(d(1:end,1));
gc = diff(logc);
ge = diff(loge);
gd = diff(logd);
logdc = logd -logc;
logec = loge -logc;
L = 5;
n = 2;
T = length(logec)-L;
b = (0.9:0.001:1)';
theta = (5:5:100)';

% Step 1; Estimate models
B  = {}; % coefficient
S  = {}; % var-cov matrix of error
et = {}; % error

% Model 1: VAR in levels with/without GC; y= [logc loge] 
y{1,1}  = [logc';loge'];
[B{1,1} S{1,1} et{1,1}] = MLEVAR(y,L,0);
clear y;
y{1,1} = {logc'};
y{2,1} = {loge'};
LL{1,1} = L;
LL{2,1} = L; 
trend{1,1} = 0;
trend{2,1} = 0;
[B{1,2} S{1,2} et{1,2}] = MLEVARGC(y,LL,trend);

% Model 2: VAR with cointegration, with/without GC, y = [gc c-e]
clear y;
y{1,1} = gc';
y{2,1} = logec';
[B{2,1} S{2,1} et{2,1}] = MLEVAR(y, [L-1;L], [0;0]);
clear y;
y{1,1} = {gc'};
y{2,1} = {logec'};
LL{1,1} = L-1;
LL{2,1} = L; 
[B{2,2} S{2,2} et{2,2}] = MLEVARGC(y, LL, trend);

% Model 3: VAR in first difference, with/without GC; y= [gc ge] 
clear y;
y{1,1}  = [gc';ge'];
[B{3,1} S{3,1} et{3,1}] = MLEVAR(y,L-1,0);
clear y;
y{1,1} = {gc'};
y{2,1} = {ge'};
LL{1,1} = L-1;
LL{2,1} = L-1; 
[B{3,2} S{3,2} et{3,2}] = MLEVARGC(y,LL,trend);

% Calculate loglikelihood of each models
T = length(logc)-L; % sample size
n = 2; % number of variables in each VAR
logL = zeros(3,2);
for i = 1:3
    for j = 1:2
        logL(i,j) = (-T/2)*(n*log(2*pi)+n+log(det(S{i,j})));
    end;
end;


% Step 2; rearrange the systems to VAR(1)
% factorize the variance-covariance matrix to define different shocks
% V1: low triagular
% V2: gamma1 (limiting response) is lower triagular, i.e. permanent and
% temporary shocks
clear LL;
LL = [L;L;L-1]; % number of lags in each VAR
for i = 1:3
    for j = 1:2
        mx{i,j}  = [B{i,j}(:,1);zeros((LL(i)-1)*n, 1)]; % constant
        A{i,j}   = [B{i,j}(:,3:end)]; % coefficient on lags
        A1{i,j}  = DISCSUM(A{i,j},1,LL(i)); % discounted sum of matrix
        V{1,i,j} = chol(S{i,j})'; % cholusky decomposition of variance-cov
        temp     = inv(eye(n)-A1{i,j});
        tempV    = (chol(temp*S{i,j}*temp'))';
        V{2,i,j} = (eye(n)-A1{i,j})*tempV; % Blanchard-Quach Decomposition
        G{i,j}   = [A{i,j};[eye((LL(i)-1)*n) zeros((LL(i)-1)*n,n)]];
        H{1,i,j} = [V{1,i,j}; zeros((LL(i)-1)*n, n)];
        H{2,i,j} = [V{2,i,j}; zeros((LL(i)-1)*n, n)];
        uc{i,j}  = [1 0 zeros(1,(LL(i)-1)*n)];  
        ue{i,j}  = [0 1 zeros(1,(LL(i)-1)*n)];         
    end;
end;
    
% Calculate Uc, mc and gamma0
for i = 1:3
    for j= 1:2
        mc{i,j} = uc{i,j}*inv(eye(size(G{i,j}))-G{i,j})*mx{i,j};
        switch i 
            case 1
                Uc{i,j} = uc{i,j}*(G{i,j}-eye(size(G{i,j})));
                Ue{i,j} = ue{i,j}*(G{i,j}-eye(size(G{i,j})));
                mc{i,j} = 0;
            case 2
                Uc{i,j} = uc{i,j}*G{i,j};
                Ue{i,j} = ue{i,j}*(G{i,j}-eye(size(G{i,j})))+uc{i,j}*G{i,j};                
            otherwise
                Uc{i,j} = uc{i,j}*G{i,j};
                Ue{i,j} = ue{i,j}*G{i,j};
        end;              
        for k =1:2
            gamma0{k,i,j} = uc{i,j}*H{k,i,j};
        end;
    end;
end;

% calculate gamma(beta) and impulse response
TT = 80;
for i = 1:3 % model index
    for j = 1:2 % with or without GC
        for k = 1:2 % different shock identification
            gammab{k,i,j} = []; % discounted response of consumption
            for m = 1:length(b)
                if i ==1
                    tempG = inv(eye(size(G{i,j}))-b(m)*G{i,j});
                    gammab{k,i,j} = [gammab{k,i,j}; (1-b(m))*uc{i,j}*tempG*H{k,i,j}];
                else
                    tempG = inv(eye(size(G{i,j}))-b(m)*G{i,j});
                    gammab{k,i,j} = [gammab{k,i,j}; uc{i,j}*tempG*H{k,i,j}];
                end;
            end;
            gb{k,i,j} = sqrt(sum(gammab{k,i,j}.^2,2));
%            IR{k,i,j} = [];
            IR0{k,i,j} = zeros(TT,n*n);
            IR0{k,i,j}(1,:) = [gamma0{k,i,j} (ue{i,j}*H{k,i,j})];
            for t = 1:TT-1           
 %               temp1 = temp1 + [(Uc{i,j}*G{i,j}^(t-1)*H{k,i,j})';(Ue{i,1}*G{i,1}^(t-1)*H{k,i,j})'];
%                IR{k,i,j} = [IR{k,i,j} temp1];
               IR0{k,i,j}(t+1,:) = [(Uc{i,j}*G{i,j}^(t-1)*H{k,i,j}) (Ue{i,1}*G{i,1}^(t-1)*H{k,i,j})];                
            end;   
            IR{k,i,j} = cumsum(IR0{k,i,j})';
        end;
    end;
end;



% PART III: produce the figures in Hansen, Heaton and Li (2007)
% plot impulse response of consumption to permanent and temporary shocks
% for cointegrated model without GC restriction
% Figure 3  
figure(1)
plot(0:TT-1,IR{2,2,1}(1,:),'-',0:TT-1,-IR{2,2,1}(2,:),'-.');
ylim([-0.005,0.015]);
print -depsc figure3;

% plot norm of lambda(beta) for models without GC restriction
% Figure 5 
figure(2)
plot(b,gb{2,2,1},'-',b,gb{k,3,1},'--',b,gb{2,1,1},'-.');
ylim([0 0.012]);
xlabel('\beta');
xlim([0.9457,1]);
print -depsc figure5;