%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright Hansen, Heaton, Li (2007)                                   
% Last revised May 2008
% 1. Load data on consumption and earning data from NIPA, 
% 2. Load data on returns with/without divdiend from CRSP 
% 3. Construct quarterly cash flow series 
% Data src: NIPA Table Section1All_xls.xls from www.bea.gov
%           Portfolio formed on BE/ME from Kenneth French's website
%           Market Portfolio, Tbill rate and CPI from CRSP at WRDS 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;
close all;
%% define data range
T1 = 1; % NIPA quarterly data goes to 2008Q1 except for earning data, so we use data up to 2007Q4
mT1 = 0; % return data goes to 2007Dec, we use data up to 2006Dec
aT1 = 0;
date = (1947:0.25:2007.75)';
%% NIPA data quarterly 1947Q1-2007Q4
temp = xlsread('E:\work\Data\NIPAdata\NIPA\Section1ALL_xls.xls','10104 Qtr');
nipa10104 = temp(3:end,[1 4:end-T1])';
clear temp;
temp = xlsread('E:\work\Data\NIPAdata\NIPA\Section1ALL_xls.xls','10105 Qtr');
nipa10105 = temp(3:end,[1 4:end-T1])';
clear temp;
temp = xlsread('E:\work\Data\NIPAdata\NIPA\Section1ALL_xls.xls','11200 Qtr');
nipa11200 = temp([3:27 29:33 35:end], [1 4:end-T1])';
clear temp;
CD      = nipa10105(2:end,3); % durables (not used)
CN      = nipa10105(2:end,4); % nondurables
CS      = nipa10105(2:end,5); % services
PCD     = nipa10104(2:end,3)/100; % price index for durables (not used)
PCN     = nipa10104(2:end,4)/100; % price index for nondurables
PCS     = nipa10104(2:end,5)/100; % price index for services
NIPAE   = nipa11200(2:end,13);% corporate before-tax profit with IVA and CCadj
NIPAE2  = nipa11200(2:end,15);% corporate after-tax profit with IVA and CCadj(not used)
NIPAD   = nipa11200(2:end,16);% net dividend (not used)
PCE     = (PCN.*CN+PCS.*CS)./(CN+CS);
C       = CN+CS; % nominal consumption
c       = C./PCE;   % real consumption
e       = NIPAE./PCE;     % real corporate earnings 
e2      = NIPAE2./PCE;
nipad   = NIPAD./PCE; % real dividends
Inflation = log(PCE(2:end,1)./PCE(1:end-1,1)); % 1947Q2-2007Q4
clear nipa1*;

%% Return data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get data on riskfree rate and cpi; 
% src: wrds\crsp\Indices and Deciles (A)\Treasury and Inflation\US Treasury and Inflation - Monthly, 
% 1925dec-current
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
load 'E:\work\data\crsp\CRSP_TR_mon.txt'; % 90-day and 30-day Tbill return
%sr0 = 242; % 1946Jan
sr0 = 8; % 1926Jul;
sr1 = length(CRSP_TR_mon)-mT1; %2007Dec
cpiret = CRSP_TR_mon(sr0:sr1,11); % inflation rate of CPI
rf = CRSP_TR_mon(sr0:sr1,10); % monthly 30 day T-bill return;
%rf = CRSP_TR_mon(sr0:sr1,9); % monthly 90 day T-bill return, there is spike in T90 in 1946Aug;
datemon = floor(CRSP_TR_mon(sr0,1)/10000):1/12:floor(CRSP_TR_mon(sr1,1)/10000)+11/12;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get data on market portfolio; 
% src: wrds\crsp\monthly stocks\Monthly Stock - Market Indices (NYSE/AMEX/NASDAQ)
% 1925Dec-current
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
load 'E:\work\data\crsp\marketindex_mon.txt';  
%sr0 = 242; % 1946Jan
sr0 = 8; % 1926Jul;
sr1 = length(marketindex_mon)-mT1; %2007Dec
mret = marketindex_mon(sr0:sr1,2); % value-weighted return with dividend
mretx = marketindex_mon(sr0:sr1,3); % value-weighted return without dividend
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get data on 5 BM portfolio; 
% src: Kennth French website, 1926July-2007dec
% portfolio formed on BE/ME with and without dividends
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
x       = xlsread('E:\work\Data\FamaFrench\FFBMPORT.xls','vwret');
ret     = [x(1:end-mT1,6:10)./100 mret]; % 5 BM portfolio & market;
clear x;
x       = xlsread('E:\work\Data\FamaFrench\FFBMPORT.xls','vwretx');
retx    = [x(1:end-mT1,6:10)./100 mretx]; % 5 BM portfolio & market;
clear x;
x       = xlsread('E:\WORK\DATA\FamaFrench\FFBMPORT.xls','size'); % average firm size
fsize   = x(1:end-mT1,6:10); % 5 BM portfolio;
clear x;
x       = xlsread('E:\WORK\DATA\FamaFrench\FFBMPORT.xls','fn'); % average firm number
fn      = x(1:end-mT1,6:10); % 5 BM portfolio;
clear x;
x       = xlsread('E:\WORK\DATA\FamaFrench\FFBMPORT.xls','vwbm'); % value-weighted average BE/ME
vwbm    = x(22:end-aT1,6:10); % annual 1947-
datea   = x(1:end-aT1,1);
clear x;
x       = xlsread('E:\WORK\DATA\FamaFrench\FFBMPORT.xls','beme'); % sum(BE)/sum(ME)
beme    = x(22:end-aT1,6:10); % annual 1947-
clear x;

% calcualte annual firm size used to calcualte value-weighted average
% use every year July data
[tm N] = size(fsize);
ta = floor(tm/12)+1; % 1926-2007
for i = 1:ta
    fsizeann(i,:) = fsize((i-1)*12+1,:);
    fnann(i,:) = fn((i-1)*12+1,:);
end;

fmv = fsizeann(22:end,:).*fnann(22:end,:);

% calculate BM ratio for mkt portfolio
bememkt = sum(beme.*fmv,2)./sum(fmv,2); % sum(BE)/sum(ME)
vwbmmkt = sum(vwbm.*fmv,2)./sum(fmv,2); % value-weighted average of BM ratio
beme = [beme bememkt];
vwbm = [vwbm vwbmmkt];

%% Construct Cash flows
% Ref: HHL(2005), Nan Li's website and Hansen, Heaton and Li (2005)
[tm N] = size(ret);
DY = ret - retx; % dividend yields, D_t/P_(t-1)
Pt0 = zeros(tm,N);   % P_t/P_0
Pt0(1,:) = 1+retx(1,:); 
DP0  = zeros(tm,N);   % D_t/P_0
DP0(1,:) = DY(1,:); 
for i = 2:tm
    Pt0(i,:) = Pt0(i-1,:).*(1+retx(i,:));
    DP0(i,:) = Pt0(i-1,:).*DY(i,:);  
end;
%PD = Pt0./DP0;

% monthly to quarterly
tq = floor(tm/3); % 1926Q2-2006Q4
dateq = linspace(1926.5,2006.75,tq)'; % date index for quarterly data
for i =1:tq
    Rf(i,1) = sum(log(1+rf((i-1)*3+1:i*3,1))); % log risk free rate
    R(i,:)  = sum(log(1+ret((i-1)*3+1:i*3,:))); % log return
    tempQDP0(i,:) =  sum(DP0((i-1)*3+1:i*3,:)); % absloute level
    tempQP0(i,:) = Pt0(i*3,:);
end;

% 12-mon trailing ave dividends D/P0 to get rid of seasonality.
% take simple mean gives the same result as taking the average of log(D)
% however, take log(D) has some NA values

tempAQDP0 = zeros(tq,N);
for i=1:tq;
    if i<4 
        tempAQDP0 (i,:)= mean(tempQDP0(1:i,:));
    else
        tempAQDP0 (i,:)= mean(tempQDP0(i-3:i,:));
    end;
end;

s2 = 83; % 1947Q1
% Initialize such the dividend for every portfolio in 1947Q1 is same as
% personal consumption of nondurables and services (in nominal value)
P0 = C(1,1)*ones(1,N)./tempAQDP0(s2,:); 

% other initialization
% Initialize the dividends such that aggregate dividend is same as NIPA
% earning in 1947 Q1 and in proportion of the dividend in 1947Q1
%P0 = (NIPAE(1,1)/tempAQDP0(s2,6))*ones(1,N);
% Market Pt in 1947Q1
%P0 = (NIPAE(1,1)*(tempAQDP0(s2,:)./tempAQDP0(s2,end))); 

T = length(NIPAE);
QP   = tempQP0(s2:end,:).*kron(P0, ones(T,1)); 
QD = tempAQDP0(s2:end,:).*kron(P0, ones(T,1));
QPD = QP./QD; % P_t/D_t
d = QD./kron(PCE, ones(1,N));

r = R(s2+1:end,:)- Inflation*ones(1,N); % real return
rrf = Rf(s2+1:end,1) - Inflation; % real risk free rate
rex = R - Rf*ones(1,N); % excess return in excess of risk free rate
sharpe1 = mean(exp(r)-exp(rrf)*ones(1,N))'./std(exp(r)-exp(rrf)*ones(1,N))'; %sharper ratio of excess returns
sharpe2 = mean(rex(s2:end,:))'./std(rex(s2:end,:))'; % sharpe ratios of excess log return

save longrun.mat;

