%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright Hansen, Heaton, Li (2007)                                   
% Last revised May 2008
% This program estimates one-period expected return using VAR in [ct-c(t-1), et-ct] 
% and produces Table 1 and Figure 1. 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all;
close all;
load longrun.mat;
N = size(r,2); % number of portfolios
% r is the log real return
logc = log(c(2:end,1));
loge = log(e(2:end,1));
gc = diff(logc);
ge = diff(loge);
logec = loge -logc;
L = 5;
n = 3;
T = length(logec)-L;

% Step 1; Estimate model: VAR with cointegration, return does not GC gc and logec,
% Model 1: VAR in with GC; y= [gc e-c r] 
y1{1,1}     = gc';
y1{2,1}     = logec';
y{1,1}      = y1;
LL{1,1}     = [L-1;L]; 
LL{2,1}     = L;
trend{1,1}  = [0;0];
trend{2,1}  = 0;

T = length(logc)-L; % sample size
n = 3; % number of variables in each VAR
logL = zeros(N,1); % log-likelihood

for i = 1:N
    y{2,1}  = {r(:,i)'};
    [B{i} S{i} et{i}] = MLEVARGC(y, LL, trend);
    logL(i,1) = (-T/2)*(n*log(2*pi)+n+log(det(S{i})));
end;

% Step 2; rearrange the systems to VAR(1)
ur  = [0 0 1 zeros(1,(L-1)*n)]; 
for i = 1:N
    mx{i}  = [B{i}(:,1);zeros((L-1)*n, 1)]; % constant
    A{i}   = [B{i}(:,3:end)]; % coefficient on laggs
    G{i}   = [A{i};[eye((L-1)*n) zeros((L-1)*n,n)]];
    er(i,1) = ur*inv(eye(n*L)-G{i})*mx{i};
end

table1 = [er'*400; % one-period expected return constructed from VAR
          mean(r)*400; % sample average quarterly real return in annual unit
          mean(vwbm);
          mean(QPD)/4]
figure(1)
plot(date, log(d(:,5)./c),'-',date,log(d(:,6)./c),'--',date,log(d(:,1)./c),'-.');
%title('Cash Flows Relative to Consumption for Five B/M Portfolios and Market Portfolio')
xlim([1947 2008]);
ylim([-1.5 1.5]);
legend('Value Portfolio', 'Market Portfolio', 'Growth Portfolio',2)

